package Beispiele::iX::Mbox;


use strict;
use Digest::MD5 qw(md5_hex);
use Fcntl qw(:flock :seek);


# -----  Konstruktor  -----
# * Para: mbox Dateiname
# * mbox wird geffnet, exklusiv gelockt und geparsed
# * Objekt: $this->{fh => <Dateihandle>, msgs => [<Nachrichten>]}
#
sub new
{   my ($class, $file) = @_;

    open my $fh, "+<", $file   or return;
    flock $fh, LOCK_EX|LOCK_NB or return;

    my $this = bless {fh => $fh}, $class;

    seek $fh, 5, SEEK_SET or return;    # beginnendes "From_" schlucken
    local $/;
    for (my $last = 0; !$last;)
    {   $/ = "\n";          chomp(my $frm = <$fh>);
        $/ = "\n\n";        chomp(my $hdr = <$fh>);
        $/ = "\n\nFrom ";   my $body = <$fh>;
        if (eof $fh) { $last = 1 }
        else         { chomp $body; $body .= "\n"; }

        my @hdrLines = split /\n/, $hdr;
        push @{$this->{msgs}},
        {   from => "From $frm"                         # From_-Zeile
        ,   hdrs => 1+$#hdrLines                        # Anz. Headerzeilen
        ,   text => [@hdrLines, "", $body=~/^(.*)/mg]   # Nachricht
        ,   len  => length($hdr) + length($body)        # Lnge
        ,   uidl => md5_hex $hdr                        # id
        ,   dele => 0                                   # Lschstatus
        };
    }

    return $this;
}


# -----  dele()  -----
# * markiert eine Nachricht als zu lschen; Para: Nr. der Nachricht (ab 1)
#
sub dele
{   my ($this, $nr) = @_;
    return if $nr < 1 || $nr > $#{$this->{msgs}}+1;

    $this->{msgs}->[$nr-1]->{dele} = 1;
    return 1;
}


# -----  reset()  -----
# * markiert alle Nachrichten als ungelscht
#
sub reset { for my $msg (@{shift->{msgs}}) {$msg->{dele} = 0} }


# -----  count()  -----
# * Anzahl der nicht als gelscht markierten Nachrichten
#
sub count { scalar grep !$_->{dele} => @{shift->{msgs}} }


# -----  maxNum()  -----
# * Nummer der letzten Nachricht (zur Iteration ber alle Nachrichten)
#
sub maxNum { 1+$#{shift->{msgs}} }


# -----  isDeleted()  -----
# * gibt an, ob eine Nachricht gelscht ist
#
sub isDeleted
{   my ($this, $nr) = @_;
    return if (defined $nr && $nr < 1) || $nr > 1+$#{$this->{msgs}};

    return $this->{msgs}->[$nr-1]->{dele};
}


# -----  size()  -----
# * Gre einer Nachricht (Para: Nr. der Nachricht ab 1), oder der mbox
#   (kein Para)
#
sub size
{   my ($this, $nr) = @_;
    return if (defined $nr && $nr < 1) || $nr > 1+$#{$this->{msgs}};

    return $this->{msgs}->[$nr-1]->{len} if defined $nr;

    my $size = 0;
    for my $msg (@{$this->{msgs}}) {$size += $msg->{len} unless $msg->{dele}}
    return $size;
}


# -----  purge()  -----
# * lscht die als gelscht markierten Nachrichten; schreibt mbox Datei neu
#
sub purge
{   my $this = shift;

    my @msgs = grep !$_->{dele} => @{$this->{msgs}};
    return 1 if $#msgs == $#{$this->{msgs}};

    my $fh = $this->{fh};
    seek $fh, 0, SEEK_SET or return;

    for my $i (0..$#msgs)
    {   print "\n" unless 0 == $i;
        print join("\n", $msgs[$i]->{from}, @{$msgs[$i]->{text}}), "\n";
    }
    truncate $fh, tell $fh or return;
    @{$this->{msgs}} = @msgs;

    return 1;
}


# -----  uidl()  -----
# * gibt id einer Nachricht (Para: Nr. ab 1) zurck
#
sub uidl
{   my ($this, $nr) = @_;
    return if $nr < 1 || $nr > $#{$this->{msgs}}+1;

    my $msg = $this->{msgs}->[$nr-1];
    return $msg->{dele} ? undef : $msg->{uidl};
}


# -----  retr()  -----
# * Referenz auf Nachrichtzeilenarray; Para: Nr. der Nachricht (ab 1)
#
sub retr
{   my ($this, $nr) = @_;

    return     $nr < 1
            || $nr > 1+$#{$this->{msgs}}
            || $this->{msgs}->[$nr-1]->{dele}
            ? undef : $this->{msgs}->[$nr-1]->{text};
}


# -----  top()  -----
# * Referenz auf Array der ersten n Zeilen einer Nachricht und des Headers;
#   Para: Nr. der Nachricht (ab 1)
#
sub top
{   my ($this, $nr, $lines) = @_;
    return if $nr < 1 || $nr > 1+$#{$this->{msgs}} || $lines < 0;

    my $msg = $this->{msgs}->[$nr-1];
    return if $msg->{dele};

    my $lastLine = $lines+$msg->{hdrs}<$#{$msg->{text}}
        ? $msg->{hdrs}+$lines : $#{$msg->{text}};
    return [@{$msg->{text}}[0..$lastLine]];
}


1;
