package Beispiele::iX::POP3;

use strict;


# -----  Konstanten fr POP3-Zustnde  -----
#
use constant
{   STATE_AUTH => 1
,   STATE_TRAN => 2
};


# -----  dispatch table  -----
#
my %cmds =
(   auth => \&auth
,   dele => \&dele
,   list => \&list
,   noop => sub { "+OK\n" }
,   pass => \&pass
,   quit => \&quit
,   retr => \&retr
,   rset => \&rset
,   stat => \&stat
,   top  => \&top
,   uidl => \&uidl
,   user => \&user
);


# -----  Konstruktor  -----
# * Callbacks fr verschiedene Vorgnge als Hash
#   + mbox  => Subroutine zur Ermittlung der Mailbox aus dem Nutzernamen
#   + basic => Subroutine zur berprfung von Nutzer/Passwort
#   + quit  => Subroutine, die bei Sitzungsende ausgefhrt wird
#
sub new
{   my ($class, %paras) = @_;

    my $pop = bless
    {   state => +STATE_AUTH
    ,   mbox  => $paras{mbox}  || sub {}
    ,   basic => $paras{basic} || sub {}
    ,   quit  => $paras{quit}  || sub {}
    }, $class;

    # --- Objekt und "welcome string" zurckgeben ---
    # * todo: der "welcome string" kann von den Auth-Methoden abhngen
    return ($pop, "+OK\n");
}


# -----  client()  -----
# * Zugriff durch den Client
#
sub client
{   my $this = shift;
    my ($cmd, @paras) = split /\s+/, shift;
    $cmd = lc $cmd;

    return $cmds{$cmd} ? $cmds{$cmd}->($this, @paras)
                       : "-ERR Command not implemented\n";
}


# -----  user()  -----
# * setzt den Nutzer (basic auth)
#
sub user
{   my ($this, $user) = @_;
    return "-ERR Not in authorization state\n"
        unless STATE_AUTH == $this->{state};

    $this->{user} = $user;
    return "+OK User $user recognised\n";
}


# -----  pass()  -----
# * setzt und verifiziert Passwort des Nutzers (basic auth)
# * ermittelt die Mailbox des Nutzers
# * wechselt in den "transaction state"
#
sub pass
{   my ($this, $pass) = @_;
    return "-ERR Not in authorization state\n"
        unless STATE_AUTH == $this->{state};
    return "-ERR Specify user first\n" unless defined $this->{user};

    my $user = $this->{user};
    return "-ERR\n" unless $this->{basic}->($user, $pa);

    my $mbox = $this->{mbox}->($user);
    return "-ERR\n" unless defined $mbox;
    $this->{mbox}  = $mbox;
    $this->{state} = STATE_TRAN;

    return "+OK Welcome $user\n";
}


# -----  dele()  -----
# * markiert eine Nachricht als gelscht
#
sub dele
{   my ($this, $nr) = @_;
    return "-ERR Not in transaction state\n"
        unless STATE_TRAN == $this->{state};

    my $dele = $this->{mbox}->dele($nr);

    return defined $dele ? "+OK message $nr deleted\n" : "-ERR\n";
}


# -----  list()  -----
# * Scanlisting der Mailbox oder einer Nachricht
#
sub list
{   my ($this, $nr) = @_;
    return "-ERR Not in transaction state\n"
        unless STATE_TRAN == $this->{state};

    my $mbox = $this->{mbox};

    if (defined $nr)
    {   return "-ERR\n" unless $nr =~ /^\d+$/;

        my $size = $mbox->size($nr);
        return $size ? "+OK $nr $size\n" : "-ERR no such message\n";
    }
    else
    {   return join "\n"
        ,   "+OK"
        ,   map
            (   "$_ ".$mbox->size($_)
                => grep !$mbox->isDeleted($_)
                => 1..$mbox->maxNum()
            )
        , ".\n";
    }
}


# -----  quit()  -----
# * fhrt den quit-Callback aus und beendet die Sitzung
#
sub quit
{   my $this = shift;

    $this->{quit}->();

    return "+OK\n" if STATE_AUTH == $this->{state};

    $this->{mbox}->purge();
    return "+OK\n";
}


# -----  retr()  -----
# * gibt Nachricht aus
#
sub retr
{   my ($this, $nr) = @_;
    return "-ERR Not in transaction state\n"
        unless STATE_TRAN == $this->{state};

    my $msg = $this->{mbox}->retr($nr);
    return "-ERR no such message\n" unless defined $msg;

    for my $i (0..$#$msg) { $msg->[$i] =~ s/^\.$/\.\./ }
    return join "\n"
    ,   "+OK"
    ,   @$msg
    ,   ".\n";
}


# -----  rset()  -----
# * markiert alle Nachrichten als ungelscht
#
sub rset
{   my ($this, $nr) = @_;
    return "-ERR Not in transaction state\n"
        unless STATE_TRAN == $this->{state};

    $this->{mbox}->reset();
    return "+OK\n";
}


# -----  stat()  -----
# * Mailbox-Statistik: Anzahl ungelschter Nachrichten und Gre der Mailbox
#
sub stat
{   my $this = shift;
    return "-ERR Not in transaction state\n"
        unless STATE_TRAN == $this->{state};

    my $mbox  = $this->{mbox};
    my $count = $mbox->count();
    my $size  = $mbox->size();
    return "+OK $count $size\n";
}


# -----  top()  -----
# * gibt Header und angegebene Anzahl Zeilen einer Nachricht aus
#
sub top
{   my ($this, $nr, $lines) = @_;
    return "-ERR Not in transaction state\n"
        unless STATE_TRAN == $this->{state};

    my $msg = $this->{mbox}->top($nr, $lines);
    return "-ERR no such message\n" unless defined $msg;

    for my $i (0..$#$msg) { $msg->[$i] =~ s/^\.$/\.\./ }
    return join "\n"
    ,   "+OK"
    ,   @$msg
    ,   ".\n";
}


# -----  uidl()  -----
# * gibt eindeutige id einer Nachricht zurck
#
sub uidl
{   my ($this, $nr) = @_;
    return "-ERR Not in transaction state\n"
        unless STATE_TRAN == $this->{state};

    my $uidl = $this->{mbox}->uidl($nr);
    return defined $uidl ? "+OK $uidl\n" : "-ERR\n";
}


1;
